# Testing my Bucket.pm
# 11/03/2009 geoff mclane http://geoffair.net/mperl
unshift(@INC, 'C:/GTools/perl');
use strict;
use warnings;
use Bucket; # see Bucket.pm

require "logfile.pl" or die "Missing logfile.pl ...\n"; # my simple log file and some other utility subs

# log file stuff
my ($LF);
my $pgmname = $0;
if ($pgmname =~ /\w{1}:\\.*/) {
	my @tmpsp = split(/\\/,$pgmname);
	$pgmname = $tmpsp[-1];
}
my $outfile = "temp.$pgmname.txt";
open_log($outfile);
prt( "$pgmname ... Hello, World...\n" );

my $SG_BUCKET_SPAN = 0.125;

my $show_indexes = 0;
my $show_all = 0;

my $MIN_LON = 150;
my $MAX_LON = 154;
my $MIN_LAT = -37;
my $MAX_LAT = -30;

my ($b, $b2);
my ($ac);
my ($item1, $item2, $item3, $item4);

#sub prt($) {
#   print shift;
#}

sub show_bucket($) {
   my ($b) = shift;
   prt( "lon:lat:x:y:base_path/index = " );
   prt( $b->bucket_info() );
   prt( " center: ".$b->get_center_lon().",".$b->get_center_lat()."\n" );
   prt( "Corners:" );
   for (my $i = 0; $i < 4; $i++) {
      my ($lon,$lat) = $b->get_corner($i);
      prt( " $i:" );
      if ($i == 0) {
         prt( "BL" );
      } elsif ($i == 1) {
         prt( "BR" );
      } elsif ($i == 2 ) {
         prt( "TR" );
      } else {
         prt( "TL" );
      }
      prt( ": $lon,$lat" );
   }
   prt("\n");
   prt( "Width ".$b->get_width_m().", Height ".$b->get_height_m()." meters. " );
   my $path = $b->gen_base_path();
   my @arr = split('/',$path);
   prt( "CHUNK=".$arr[0]."\n" );
}

# attempt to get ALL buckets with this SAME BASE PATH
sub get_all_buckets_same_1X1($) {
   my ($b) = shift;
   my @bucks = ();
   my ($i, $path, $cnt, $nb, $np, $nb1, $tb, $fnd);
   $path = $b->gen_base_path();
   $cnt = 0;
   for ($i = 0; $i < 8; $i++) {
      $nb = $b->get_next_bucket($i);
      $np = $nb->gen_base_path();
      if ($path eq $np) {
         # in the same relm
         push(@bucks, $nb);
         $cnt++;
      }
   }
   # now process all the near bucket found
   $cnt = scalar @bucks;
   while ($cnt) {
      $cnt = 0;   # count of NEW buckets
      foreach $nb1 (@bucks) {
         for ($i = 0; $i < 8; $i++) {
            $nb = $nb1->get_next_bucket($i);
            $np = $nb->gen_base_path();
            if ($path eq $np) {
               # in the same relm
               if ( ! $b->buckets_equal( $b, $nb ) ) {
                  # not equal to last
                  $fnd = 0;
                  foreach $tb (@bucks) {
                     if ($b->buckets_equal( $nb, $tb ) ) {
                        $fnd = 1;
                        last;
                     }
                  }
                  if ( $fnd == 0 ) {
                     push(@bucks, $nb);
                     $cnt++;
                  }
               }
            }
         }
      }
   }
   $fnd = 0;
   foreach $tb (@bucks) {
      if ( $b->buckets_equal( $b, $tb ) ) {
         $fnd = 1;
         last;
      }
   }
   if ( $fnd == 0 ) {
      push(@bucks, $b);
      $cnt++;
   }
   return @bucks;
}

sub show_inds($) {
   my ($br) = shift;
   my @ind = ();
   my ($tb, $i, $cnt, $il);
   foreach $tb (@$br) {
      #prt( $tb->gen_index()." " );
      push(@ind, $tb->gen_index());
   }
   $cnt = 0;
   $il = -1;
   foreach $i (sort @ind) {
      if ($cnt == 0) {
         prt("$i");
         $cnt++;
      } elsif ($i == ($il + 1)) {
         $cnt++;
      } else {
         # new index seq
         if ($cnt == 1) {
            prt(" $i");
         } else {
            prt("-$il $i");
         }
         $cnt = 1;
      }
      $il = $i;   # update last
   }
   if ($cnt > 1) {
      prt("-$il");
   }
   prt("\n");
}

#################
sub parse_args {
   my (@av) = @_;
   my ($arg, $argc);
   $argc = 0;
   prt( "parse_args: ".scalar @av."\n" );
   while(@av) {
      $argc++;
      $arg = $av[0];
      if ($argc == 1) {
         $item1 = $arg;
         prt( "$argc: Set item1 = $item1\n" );
      } elsif ($argc == 2) {
         $item2 = $arg;
         prt( "$argc: Set item2 = $item2\n" );
      } elsif ($argc == 3 ) {
         $item3 = $arg;
         prt( "$argc: Set item2 = $item3\n" );
      } elsif ($argc == 4 ) {
         $item4 = $arg;
         prt( "$argc: Set item4 = $item4\n" );
      } else {
         prt( "$argc: Unknown = $arg\n" );
         last;
      }
      shift @av;
   }
}

# =================================================
# MAIN PROGRAM

$b = Bucket->new();   # constructor

$ac = scalar @ARGV;

parse_args(@ARGV);

if ($ac == 1) {
   prt( "Assume index [$item1]...\n" );
   $b->set_bucket_per_index($item1);
   if ($b->gen_index() != $item1) {
      prt( "ERROR: Not a valid index [$item1]! Nearest is ".$b->gen_index()."\n" );
      exit(1);
   }
} elsif ($ac == 2 ) {
   prt( "Assume [lon, lat] = [$item1, $item2] ...\n" );
   $b->set_bucket($item1,$item2);
} elsif ($ac == 4 ) {
   prt( "Assume min lon lat max lat lon = $item1, $item2, $item3, $item4\n" );
   $b->set_bucket($item1,$item2);
   $b2 = Bucket->new();
   $b2->set_bucket($item3,$item4);
   if ( $b->buckets_equal($b, $b2) ) {
      prt( "Buckets are EQUAL.\n" );
      show_bucket($b);
   } else {
      # must do sgBucketDiff(b_min, b_max, &dx, &dy);
      # to get dx and dy, then process each
      # 
      prt( "Buckets are DIFFERENT.\n" );
      my %ind_list = ();
      show_bucket($b);
      show_bucket($b2);
      my ($dy, $dx) = sgBucketDiff( $b, $b2 );
      prt( "Diff dy=$dy, dx=$dx.\n" );
      my $bcnt = 0;
      my $allcnt = 0;
      for (my $jy = 0; $jy < $dy; $jy++) {
         for (my $ix = 0; $ix < $dx; $ix++) {
            my $nb = sgBucketOffset( $item1, $item2, $ix, $jy );
            $allcnt++;
            #show_bucket($nb);
            my $ind = $nb->gen_index();
            my $c_lat = $nb->get_center_lat();
            my $c_lon = $nb->get_center_lon();
            if (($c_lat < $MAX_LAT)&&($c_lat >= $MIN_LAT)&&($c_lon < $MAX_LON)&&($c_lon >= $MIN_LON)) {
               prt( $nb->bucket_info()."\n" );
               if (defined $ind_list{$ind}) {
                  $ind_list{$ind}++;
               } else {
                  $ind_list{$ind} = 1;
               }
               $bcnt++;
            }
         }
      }
      if ($bcnt) {
         prt( "Got $bcnt of $allcnt, in range " );
      } else {
         prt( "Got NONE of $allcnt, in range " );
      }
      prt( "lon,lat MIN $MIN_LON,$MIN_LAT - MAX $MAX_LON,$MAX_LAT\n" );
      my @keys = sort (keys(%ind_list));
      my $wrap = 0;
      foreach my $ky (@keys) {
         prt( "$ky(".$ind_list{$ky}.") " );
         $wrap++;
         if ($wrap > 5) {
            prt("\n");
            $wrap = 0;
         }
      }
      if ($wrap) {
         prt("\n");
      }
   }
   close_log($outfile,1);
   unlink $outfile;
   exit(0);
} else {
   prt( "Usage: either of two inputs...\n" );
   prt( "show-bucket nnnn        assumed to be a bucket index.\n" );
   prt( "show-bucket <lon> <lat> assumed to be longitude and latitude of bucket.\n" );
   close_log($outfile,0);
   unlink $outfile;
   exit(1);
}

show_bucket($b);

if ($show_all) {
   my @bl = get_all_buckets_same_1X1($b);
   my $bcnt = scalar @bl;
   prt( "Got $bcnt buckets around...\n" );
   if ($bcnt && $show_indexes) {
      show_inds(\@bl);
   }
}

close_log($outfile,1);
unlink $outfile;
exit(0);

#// calculate the offset between two buckets
#void sgBucketDiff( const SGBucket& b1, const SGBucket& b2, int *dx, int *dy ) {
sub sgBucketDiff {
   #( const SGBucket& b1, const SGBucket& b2, int *dx, int *dy ) {
   my ($b1, $b2) = @_;
   my ($dy, $dx);
   #// Latitude difference
   # double c1_lat = b1.get_center_lat();
   # double c2_lat = b2.get_center_lat();
   # double diff_lat = c2_lat - c1_lat;
   my $c1_lat = $b1->get_center_lat();
   my $c2_lat = $b2->get_center_lat();
   my $diff_lat = $c2_lat - $c1_lat;

#ifdef HAVE_RINT
#    *dy = (int)rint( diff_lat / SG_BUCKET_SPAN );
#else
    if ( $diff_lat > 0 ) {
	   $dy = int( ($diff_lat / $SG_BUCKET_SPAN) + 0.5 );
    } else {
	   $dy = int( ($diff_lat / $SG_BUCKET_SPAN) - 0.5 );
    }
#endif

    #// longitude difference
    #double diff_lon=0.0;
    #double span=0.0;
    my $diff_lon = 0.0;
    my $span = 0.0;

    #SGBucket tmp_bucket;
    #// To handle crossing the bucket size boundary
    #//  we need to account for different size buckets.
    #if ( sg_bucket_span(c1_lat) <= sg_bucket_span(c2_lat) )
    if ( $b1->bucket_span($c1_lat) <= $b2->bucket_span($c2_lat) ) {
       $span = $b1->bucket_span($c1_lat);
    } else {
   	$span = $b2->bucket_span($c2_lat);
    }

    $diff_lon = $b2->get_center_lon() - $b1->get_center_lon();

    if ($diff_lon < 0.0) {
       $diff_lon -= ($b1->get_width()*0.5) + ($b2->get_width()*0.5) - $span;
    } else {
       $diff_lon += ($b1->get_width()*0.5) + ($b2->get_width()*0.5) - $span;
    }


#ifdef HAVE_RINT
#    *dx = (int)rint( diff_lon / span );
#else
    if ( $diff_lon > 0 ) {
	   $dx = int( ($diff_lon / $span) + 0.5 );
    } else {
	   $dx = int( ($diff_lon / $span) - 0.5 );
    }
#endif
   return $dy,$dx
}

#// find the bucket which is offset by the specified tile units in the
#// X & Y direction.  We need the current lon and lat to resolve
#// ambiguities when going from a wider tile to a narrower one above or
#// below.  This assumes that we are feeding in
#SGBucket sgBucketOffset( double dlon, double dlat, int dx, int dy ) {
sub sgBucketOffset {
   my ($dlon, $dlat, $dx, $dy) = @_;
   my $result = Bucket->new();   # constructor
   #SGBucket result( dlon, dlat );
   $result->set_bucket( $dlon, $dlat );
   #double clat = result.get_center_lat() + dy * SG_BUCKET_SPAN;
   my $clat = $result->get_center_lat() + $dy * $SG_BUCKET_SPAN;

    #// walk dy units in the lat direction
    $result->set_bucket( $dlon, $clat );

    #// find the lon span for the new latitude
    my $span = $result->bucket_span( $clat );

    #// walk dx units in the lon direction
    my $tmp = $dlon + $dx * $span;
    while ( $tmp < -180.0 ) {
	   $tmp += 360.0;
    }
    while ( $tmp >= 180.0 ) {
   	$tmp -= 360.0;
    }

    $result->set_bucket( $tmp, $clat );

    return $result;
}


# eof - show-bucket.pl
